@@ -18,7 +18,34 @@ c     normals model. In this routine, inference is based on the
 c     Polya urn representation of the Dirichlet process. The no-gaps
 c     algorithm is used.
 c
-c     Author: Alejandro Jara.
+c     Copyright: Alejandro Jara Vallejos, 2006
+c
+c     This program is free software; you can redistribute it and/or modify
+c     it under the terms of the GNU General Public License as published by
+c     the Free Software Foundation; either version 2 of the License, or (at
+c     your option) any later version.
+c
+c     This program is distributed in the hope that it will be useful, but
+c     WITHOUT ANY WARRANTY; without even the implied warranty of
+c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+c     General Public License for more details.
+c
+c     You should have received a copy of the GNU General Public License
+c     along with this program; if not, write to the Free Software
+c     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+c
+c     The author's contact information:
+c
+c     Alejandro Jara Vallejos
+c     Biostatistical Centre
+c     Katholieke Universiteit Leuven
+c     U.Z. Sint-Rafal
+c     Kapucijnenvoer 35
+c     B-3000 Leuven
+c     Voice: +32 (0)16 336892 
+c     Fax  : +32 (0)16 337015 
+c     URL  : http://student.kuleuven.be/~s0166452/
+c     Email: Alejandro.JaraVallejos@med.kuleuven.be
 c
 c---- Data -------------------------------------------------------------
 c 
@@ -222,7 +249,9 @@ c+++++Working space
       real*8 workmh1(nvar*(nvar+1)/2),workmh2(nvar*(nvar+1)/2)
       real*8 workv1(nvar),workv2(nvar),workv3(nvar)
       real*8 ywork(nvar)
-      
+
+c+++++CPU time
+      real*8 sec00,sec0,sec1,sec
       
 c++++ Define parameters
 
@@ -246,7 +275,7 @@ c++++ set random number generator
       seed2=seed(2)
       seed3=seed(3)
 
-      call setrand(seed1,seed2,seed3)
+      call setall(seed1,seed2)
      
 c++++ cluster structure
 
@@ -263,6 +292,9 @@ c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
       skipcount=0
       dispcount=0
       nscan=nburn+(nskip+1)*(nsave)
+
+      call cpu_time(sec0)
+      sec00=0.d0
       
       do iscan=1,nscan
 
@@ -867,8 +899,11 @@ c+++++++++++++ cpo and save samples
 c+++++++++++++ print
                skipcount = 0
                if(dispcount.ge.ndisplay)then
-c                  call intpr("isave",5,isave,1)
-                  tmp1=sprint(isave,nsave)                  
+                  call cpu_time(sec1)
+                  sec00=sec00+(sec1-sec0)
+                  sec=sec00
+                  sec0=sec1
+                  tmp1=sprint(isave,nsave,sec)
                   dispcount=0
                end if   
             end if
